// lwckmap.cpp
//
// lwmap(8) map check utility
//
//   (C) Copyright 2016 Fred Gleason <fredg@paravelsystems.com>
//
//   All Rights Reserved.
//

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <QApplication>
#include <QCoreApplication>

#include <sy/sycmdswitch.h>
#include <sy/syconfig.h>
#include <sy/syprofile.h>

#include "config.h"
#include "livewire_map.h"
#include "lwckmap.h"

MainObject::MainObject(QObject *parent)
  : QObject(parent)
{
  QString map_errs;

  //
  // Process Arguments
  //
  SyCmdSwitch *cmd=
    new SyCmdSwitch(qApp->argc(),qApp->argv(),"lwckmap",VERSION,LWCKMAP_USAGE);
  if(cmd->keys()==0) {
    fprintf(stderr,"lwckmap: missing argument\n");
    exit(256);
  }
  for(unsigned i=0;i<(cmd->keys()-1);i++) {
    if(cmd->key(i)=="--verbose") {
      ck_verbose=true;
      cmd->setProcessed(i,true);
    }
    if(!cmd->processed(i)) {
      fprintf(stderr,"lwckmap: unknown option\n");
      exit(256);
    }
  }

  //
  // Check Map
  //
  ck_map=new LiveWireMap();
  if(!ck_map->load(cmd->key(cmd->keys()-1),&map_errs)) {
    fprintf(stderr,"%s\n",(const char *)map_errs.toUtf8());
    exit(256);
  }
  if(!ck_map->validate(&map_errs)) {
    fprintf(stderr,"%s\n",(const char *)map_errs.toUtf8());
    exit(256);
  }
  exit(0);
}


int main(int argc,char *argv[])
{
  QCoreApplication a(argc,argv);
  new MainObject();
  return a.exec();
}
