// lwmap.h
//
// lwmap(8) map utility
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   All Rights Reserved.
//

#ifndef LWMAP_H
#define LWMAP_H

#include <stdint.h>

#include <map>
#include <vector>

#include <QObject>
#include <QTimer>

#include <sy/symcastsocket.h>
#include <sy/sylwrp_client.h>

#include "livewire_map.h"

#define LWMAP_USAGE "[options]\n"
#define LWMAP_DEFAULT_SCAN_DURATION 25000
#define LWMAP_CONNECTION_RETRY_LIMIT 2

class MainObject : public QObject
{
 Q_OBJECT;
 public:
  MainObject(QObject *parent=0);
  void startNodeProcessing();

 private slots:
  void readyReadData();
  void scanTimeoutData();
  void garbageTimeoutData();
  void connectionTimeoutData();
  void nodeConnectedData(unsigned id,bool state);
  void nodeErrorData(unsigned id,QAbstractSocket::SocketError err);

 private:
  int ProcessUseNodeList(const QString &filename);
  int ProcessSkipNodeList(const QString &filename);
  void PurgeSkippedNodes();
  void DumpNodeList();
  void SortAddresses(std::vector<uint32_t> &addrs);
  void LoadMatrix(int mtxnum);
  void Verbose(const QString &msg);
  bool map_verbose;
  QHostAddress map_interface_address;
  QString map_output_map;
  QString map_node_password;
  bool map_no_off_source;
  bool map_scan_only;
  int map_scan_duration;
  SyMcastSocket *map_advert_socket;
  QTimer *map_scan_timer;
  QTimer *map_garbage_timer;
  QTimer *map_connection_timer;
  SyLwrpClient *map_lwrp;
  std::vector<uint32_t> map_node_addresses;
  std::vector<uint32_t> map_skip_node_addresses;
  LiveWireMap *map_map;
  int map_matrix_number;
  int map_max_nodes;
  unsigned map_current_id;
  unsigned map_retry_count;
};


#endif  // LWMAP_H
