// panelconfig.cpp
//
// Configuration for LWPath Panel Applets
//
//   (C) Copyright 2016 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as
//   published by the Free Software Foundation; either version 2 of
//   the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <sy/syprofile.h>

#include "panelconfig.h"

PanelConfig::PanelConfig()
{
  conf_server_hostname="127.0.0.1";
  conf_server_port=9500;
  conf_server_username="admin";
  conf_server_password="";
  conf_matrix_number=1;
}


QString PanelConfig::serverHostname() const
{
  return conf_server_hostname;
}


uint16_t PanelConfig::serverPort() const
{
  return conf_server_port;
}


QString PanelConfig::serverUsername() const
{
  return conf_server_username;
}


QString PanelConfig::serverPassword() const
{
  return conf_server_password;
}


unsigned PanelConfig::matrixNumber() const
{
  return conf_matrix_number;
}


unsigned PanelConfig::panelQuantity() const
{
  return conf_default_policies.size();
}


unsigned PanelConfig::panelOutput(unsigned panel) const
{
  return conf_outputs[panel];
}


QString PanelConfig::panelName(unsigned panel) const
{
  return conf_names[panel];
}


bool PanelConfig::allowSource(unsigned panel,unsigned srcnum) const
{
  if(panel>=conf_default_policies.size()) {
    return false;
  }
  for(unsigned i=0;i<conf_exceptions[panel].size();i++) {
    if(conf_exceptions[panel][i]==srcnum) {
      return !conf_default_policies[panel];
    }
  }
  return conf_default_policies[panel];
}


bool PanelConfig::load(const QString &filename)
{
  SyProfile *p=new SyProfile();
  if(!p->setSource(filename)) {
    return false;
  }
  conf_server_hostname=p->stringValue("Global","ServerHostname","127.0.0.1");
  conf_server_port=p->intValue("Global","ServerPort",9500);
  conf_server_username=p->stringValue("Global","ServerUsername","admin");
  conf_server_password=p->stringValue("Global","ServerPassword","");
  conf_matrix_number=p->intValue("Global","MatrixNumber",1);

  int count=0;
  QString section=QString().sprintf("Panel%d",count+1);
  bool ok=false;
  QString policy=p->stringValue(section,"Default","Allow",&ok).toLower();
  while(ok) {
    conf_default_policies.push_back(policy=="allow");
    conf_outputs.push_back(p->intValue(section,"Output"));
    conf_names.push_back(p->stringValue(section,"Name"));
    std::vector<unsigned> exceptions;
    conf_exceptions.push_back(exceptions);

    int exeps=0;
    QString tag=QString().sprintf("ExceptSource%d",exeps+1);
    int srcnum=p->intValue(section,tag,0,&ok);
    while(ok) {
      conf_exceptions.back().push_back(srcnum);
      exeps++;
      tag=QString().sprintf("ExceptSource%d",exeps+1);
      srcnum=p->intValue(section,tag,0,&ok);
    }
    count++;
    section=QString().sprintf("Panel%d",count+1);
    policy=p->stringValue(section,"Default","Allow",&ok).toLower();
  }

  return true;
}
