// livewire_gpio.cpp
//
// LiveWire GPIO record
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   All Rights Reserved.
//

#include <QObject>

#include "livewire_gpio.h"

LiveWireGpio::LiveWireGpio(unsigned number,const QHostAddress &addr,
			   unsigned slot)
{
  gpio_number=number;
  gpio_host_address=addr;
  gpio_slot_number=slot;
  gpio_name=QObject::tr("OFFLINE");
  gpio_host_name=addr.toString();
  gpio_saved_code="xxxxx";
  gpio_bundle=new SyGpioBundle();
  gpio_timer=new QTimer();
  gpio_timer->setSingleShot(true);
}


LiveWireGpio::LiveWireGpio()
{
  gpio_number=0;
  gpio_slot_number=0;
  gpio_name=QObject::tr("OFFLINE");
  gpio_saved_code="xxxxx";
  gpio_bundle=new SyGpioBundle();
  gpio_timer=new QTimer();
  gpio_timer->setSingleShot(true);
}


LiveWireGpio::~LiveWireGpio()
{
  delete gpio_timer;
  delete gpio_bundle;
}


unsigned LiveWireGpio::number() const
{
  return gpio_number;
}


void LiveWireGpio::setNumber(unsigned num)
{
  gpio_number=num;
}


QHostAddress LiveWireGpio::hostAddress() const
{
  return gpio_host_address;
}


bool LiveWireGpio::setHostAddress(const QString &addr)
{
  gpio_host_address=addr;
  return !gpio_host_address.isNull();
}


unsigned LiveWireGpio::slotNumber() const
{
  return gpio_slot_number;
}


void LiveWireGpio::setSlotNumber(unsigned slotnum)
{
  gpio_slot_number=slotnum;
}


QString LiveWireGpio::name() const
{
  return gpio_name;
}


void LiveWireGpio::setName(const QString &str)
{
  gpio_name=str;
}


QString LiveWireGpio::hostName() const
{
  return gpio_host_name;
}


void LiveWireGpio::setHostName(const QString &str)
{
  gpio_host_name=str;
}


SyGpioBundle *LiveWireGpio::bundle() const
{
  return gpio_bundle;
}


QHostAddress LiveWireGpio::sourceAddress() const
{
  return gpio_source_address;
}


int LiveWireGpio::sourceSlot() const
{
  return gpio_source_slot;
}


void LiveWireGpio::setSourceAddress(const QHostAddress &s_addr,int s_slot)
{
  gpio_source_address=s_addr;
  gpio_source_slot=s_slot;
}


QTimer *LiveWireGpio::timer()
{
  return gpio_timer;
}


QString LiveWireGpio::savedCode() const
{
  return gpio_saved_code;
}


void LiveWireGpio::setSavedCode(const QString &code)
{
  QString new_code="";

  for(int i=0;i<SWITCHYARD_GPIO_BUNDLE_SIZE;i++) {
    if(code.mid(i,1)=="x") {
      if(gpio_saved_code.mid(i,1)=="x") {
	new_code+="x";
      }
      else {
	new_code+=gpio_saved_code+=gpio_saved_code.mid(i,1);
      }
    }
    else {
      new_code+=code.mid(i,1);
    }
  }
  gpio_saved_code=new_code;
}


void LiveWireGpio::clearSavedCode()
{
  gpio_saved_code="xxxxx";
}
