// matrix.cpp
//
// Abstract base class for switcher matrices.
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "config.h"
#include "matrix.h"

Matrix::Matrix(Type type,int mid,SyProfile *config,QObject *parent)
  : QObject(parent)
{
  matrix_type=type;
  matrix_id=mid;
  matrix_name=Matrix::typeString(type);

  matrix_clip_threshold=
    config->intValue("Global","ClipThreshold",LWPATH_DEFAULT_CLIP_THRESHOLD);
  matrix_clip_timeout=
    config->intValue("Global","ClipTimeout",LWPATH_DEFAULT_CLIP_TIMEOUT);
  matrix_silence_threshold=
    config->intValue("Global","SilenceThreshold",
		     LWPATH_DEFAULT_SILENCE_THRESHOLD);
  matrix_silence_timeout=
    config->intValue("Global","SilenceTimeout",LWPATH_DEFAULT_SILENCE_TIMEOUT);
}


Matrix::Type Matrix::type() const
{
  return matrix_type;
}


int Matrix::matrixId() const
{
  return matrix_id;
}


unsigned Matrix::nodeQuantity(int mid) const
{
  return 0;
}


bool Matrix::nodeOnline(int mid,unsigned node) const
{
  return false;
}


QString Matrix::nodeHostName(int mid,unsigned node) const
{
  return QString();
}


QHostAddress Matrix::nodeHostAddress(int mid,unsigned node) const
{
  return QHostAddress();
}


QString Matrix::nodeDeviceName(int mid,unsigned node) const
{
  return QString();
}


int Matrix::clipThreshold() const
{
  return matrix_clip_threshold;
}


int Matrix::clipTimeout() const
{
  return matrix_clip_timeout;
}


int Matrix::silenceThreshold() const
{
  return matrix_silence_threshold;
}


int Matrix::silenceTimeout() const
{
  return matrix_silence_timeout;
}


QString Matrix::matrixName(int mid) const
{
  return matrix_name;
}


void Matrix::setMatrixName(const QString &str)
{
  matrix_name=str;
}


unsigned Matrix::matrixQuantity() const
{
  return 1;
}


unsigned Matrix::gpioCodeSize(int mid) const
{
  return 0;
}

QString Matrix::inputName(int mid,unsigned input) const
{
  return tr("Input")+QString().sprintf(" %u",input+1);
}


unsigned Matrix::inputSlotNumber(int mid,unsigned input) const
{
  return 0;
}


QString Matrix::inputHostName(int mid,unsigned input) const
{
  return QString("n/a");
}


QHostAddress Matrix::inputHostAddress(int mid,unsigned input) const
{
  return QHostAddress();
}


QHostAddress Matrix::inputSourceAddress(int mid,unsigned input) const
{
  return QHostAddress();
}


int Matrix::inputSourceSlot(int mid,unsigned input) const
{
  return -1;
}


unsigned Matrix::gpiQuantity(int mid) const
{
  return 0;
}


QString Matrix::gpiCode(int mid,unsigned input) const
{
  return QString("");
}


void Matrix::setGpiCode(int mid,unsigned input,int msecs,const QString &str)
{
}


QString Matrix::outputName(int mid,unsigned output) const
{
  return tr("Output")+QString().sprintf(" %u",output+1);
}


unsigned Matrix::outputSlotNumber(int mid,unsigned output) const
{
  return 0;
}


QString Matrix::outputHostName(int mid,unsigned output) const
{
  return QString("n/a");
}


QHostAddress Matrix::outputHostAddress(int mid,unsigned output) const
{
  return QHostAddress();
}


unsigned Matrix::gpoQuantity(int mid) const
{
  return 0;
}


QString Matrix::gpoCode(int mid,unsigned output) const
{
  return QString("");
}


void Matrix::setGpoCode(int mid,unsigned output,int msecs,const QString &str)
{
}


QString Matrix::typeString(Type type)
{
  QString ret(tr("Unknown"));

  switch(type) {
  case Matrix::TypeLiveWire:
    ret=tr("LiveWire");
    break;

  case Matrix::TypeLast:
    break;
  }

  return ret;
}
