// matrix.h
//
// Abstract base class for switcher matrices.
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef MATRIX_H
#define MATRIX_H

#include <QHostAddress>
#include <QObject>
#include <QString>

#include <sy/syprofile.h>

class Matrix : public QObject
{
 Q_OBJECT;
 public:
 enum Type {TypeLiveWire=0,TypeLast=1};
  Matrix(Type type,int mid,SyProfile *config,QObject *parent=0);
  Matrix::Type type() const;
  int matrixId() const;
  virtual bool isReady()=0;
  int clipThreshold() const;
  int clipTimeout() const;
  int silenceThreshold() const;
  int silenceTimeout() const;
  virtual QString matrixName(int mid) const;
  void setMatrixName(const QString &str);
  virtual unsigned matrixQuantity() const;

  virtual unsigned nodeQuantity(int mid) const;
  virtual bool nodeOnline(int mid,unsigned node) const;
  virtual QString nodeHostName(int mid,unsigned node) const;
  virtual QHostAddress nodeHostAddress(int mid,unsigned node) const;
  virtual QString nodeDeviceName(int mid,unsigned node) const;
  virtual unsigned gpioCodeSize(int mid) const;
  virtual unsigned inputQuantity(int mid) const=0;
  virtual QString inputName(int mid,unsigned input) const;
  virtual unsigned inputSlotNumber(int mid,unsigned input) const;
  virtual QString inputHostName(int mid,unsigned input) const;
  virtual QHostAddress inputHostAddress(int mid,unsigned input) const;
  virtual QHostAddress inputSourceAddress(int mid,unsigned input) const;
  virtual int inputSourceSlot(int mid,unsigned input) const;
  virtual unsigned gpiQuantity(int mid) const;
  virtual QString gpiCode(int mid,unsigned input) const;
  virtual void setGpiCode(int mid,unsigned input,int msecs,const QString &str);
  virtual unsigned outputQuantity(int mid) const=0;
  virtual QString outputName(int mid,unsigned output) const=0;
  virtual unsigned outputSlotNumber(int mid,unsigned output) const;
  virtual QString outputHostName(int mid,unsigned output) const;
  virtual QHostAddress outputHostAddress(int mid,unsigned output) const;
  virtual unsigned outputCrosspoint(int mid,unsigned output)=0;
  virtual void setOutputCrosspoint(int mid,unsigned output,unsigned input)=0;
  virtual unsigned gpoQuantity(int mid) const;
  virtual QString gpoCode(int mid,unsigned output) const;
  virtual void setGpoCode(int mid,unsigned output,int msecs,const QString &str);
  static QString typeString(Type type);

 signals:
  void startupComplete();
  void nodeStatusAlarm(int mid,unsigned node,bool state);
  void inputNameChanged(int mid,unsigned input,const QString &name);
  void inputClipAlarm(int mid,unsigned input,bool state);
  void inputSilenceAlarm(int mid,unsigned input,bool state);
  void gpiChanged(int mid,unsigned input,const QString &code);
  void outputNameChanged(int mid,unsigned output,const QString &name);
  void outputCrosspointChanged(int mid,unsigned output,unsigned input);
  void outputClipAlarm(int mid,unsigned output,bool state);
  void outputSilenceAlarm(int mid,unsigned output,bool state);
  void gpoChanged(int mid,unsigned output,const QString &code);

 private:
  Matrix::Type matrix_type;
  QString matrix_name;
  int matrix_id;
  int matrix_clip_threshold;
  int matrix_clip_timeout;
  int matrix_silence_threshold;
  int matrix_silence_timeout;
};


#endif  // MATRIX_H
