// matrix_livewire.h
//
// Matrix driver for LiveWire.
//
//   (C) Copyright 2015-2016 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef MATRIX_LIVEWIRE_H
#define MATRIX_LIVEWIRE_H

#include <stdint.h>

#include <map>

#include <QSignalMapper>

#include <sy/sylwrp_client.h>

#include "livewire_map.h"
#include "matrix.h"

class MatrixLiveWire : public Matrix
{
 Q_OBJECT;
 public:
  MatrixLiveWire(int mid,SyProfile *config,QObject *parent=0);
  ~MatrixLiveWire();
  unsigned matrixQuantity() const;
  QString matrixName(int mid) const;
  bool isReady();
  unsigned nodeQuantity(int mid) const;
  bool nodeOnline(int mid,unsigned node) const;
  QString nodeHostName(int mid,unsigned node) const;
  QHostAddress nodeHostAddress(int mid,unsigned node) const;
  QString nodeDeviceName(int mid,unsigned node) const;
  unsigned gpioCodeSize(int mid) const;
  unsigned inputQuantity(int mid) const;
  QString inputName(int mid,unsigned input) const;
  unsigned inputSlotNumber(int mid,unsigned input) const;
  QString inputHostName(int mid,unsigned input) const;
  QHostAddress inputHostAddress(int mid,unsigned input) const;
  QHostAddress inputSourceAddress(int mid,unsigned input) const;
  int inputSourceSlot(int mid,unsigned input) const;
  unsigned gpiQuantity(int mid) const;
  QString gpiCode(int mid,unsigned input) const;
  void setGpiCode(int mid,unsigned input,int msecs,const QString &code);
  unsigned outputQuantity(int mid) const;
  QString outputName(int mid,unsigned output) const;
  unsigned outputSlotNumber(int mid,unsigned output) const;
  QString outputHostName(int mid,unsigned output) const;
  QHostAddress outputHostAddress(int mid,unsigned output) const;
  unsigned outputCrosspoint(int mid,unsigned output);
  void setOutputCrosspoint(int mid,unsigned output,unsigned input);
  unsigned gpoQuantity(int mid) const;
  QString gpoCode(int mid,unsigned output) const;
  void setGpoCode(int mid,unsigned output,int msecs,const QString &code);

 private slots:
  void connectedData(unsigned addr,bool state);
  void connectionErrorData(unsigned addr,QAbstractSocket::SocketError err);
  void sourceChangedData(unsigned addr,int slotnum,const SyNode &node,
			 const SySource &src);
  void audioClipAlarmData(unsigned addr,SyLwrpClient::MeterType type,
			  unsigned slotnum,int chan,bool state);
  void audioSilenceAlarmData(unsigned addr,SyLwrpClient::MeterType type,
			     unsigned slotnum,int chan,bool state);
  void destinationChangedData(unsigned addr,int slotnum,const SyNode &node,
			      const SyDestination &dst);
  void gpiChangedData(unsigned id,int slotnum,const SyNode &node,
		      const SyGpioBundle &bundle);
  void gpoChangedData(unsigned id,int slotnum,const SyNode &node,
		      const SyGpo &gpo);
  void gpiResetData(int input);
  void gpoResetData(int output);

 private:
  void LoadNodes();
  bool StartNextNode();
  LiveWireNode *GetNode(const QHostAddress &addr) const;
  LiveWireAudioport *GetInput(const QHostAddress &s_addr);
  LiveWireAudioport *GetInput(uint32_t hostaddr,unsigned slotnum) const;
  LiveWireAudioport *GetOutput(uint32_t hostaddr,unsigned slotnum) const;
  LiveWireGpio *GetGpi(uint32_t hostaddr,unsigned slotnum) const;
  LiveWireGpio *GetGpo(uint32_t hostaddr,unsigned slotnum) const;
  void LoadMap(const QString &filename);
  LiveWireMap *lw_map;
  QSignalMapper *lw_gpi_reset_mapper;
  QSignalMapper *lw_gpo_reset_mapper;
  std::map<uint32_t,SyLwrpClient *> lw_lwrps;
  std::map<uint32_t,LiveWireAudioport *> lw_input_stream_index;
  std::vector<uint32_t> lw_startup_addresses;
  unsigned lw_startup_node;
  bool lw_disable_watchdog;
  bool lw_load_complete;
  bool lw_ready;
  int lw_node_connection_timeout;
};


#endif  // MATRIX_LIVEWIRE_H
