// protocol.cpp
//
// Abstract base class for switcher control protocols.
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <syslog.h>

#include "protocol.h"

Protocol::Protocol(Type type,std::vector<Matrix *> *matrices,SyProfile *config,
		   QObject *parent)
  : QObject(parent)
{
  proto_type=type;
  proto_matrices=matrices;
}


Protocol::Type Protocol::type() const
{
  return proto_type;
}


QString Protocol::keyword(Type type)
{
  QString ret;

  switch(type) {
  case Protocol::TypeSap:
    ret="softwareauthority";
      break;

  case Protocol::TypePath:
    ret="path";
    break;

  case Protocol::TypeLast:
    break;
  }

  return ret;
}


void Protocol::nodeStatusAlarmChanged(int mid,unsigned node,bool state)
{
}


void Protocol::inputNameChanged(int mid,unsigned input,const QString &name)
{
}


void Protocol::inputClipAlarmChanged(int mid,unsigned input,bool state)
{
}


void Protocol::inputSilenceAlarmChanged(int mid,unsigned input,bool state)
{
}


void Protocol::gpiChanged(int mid,unsigned input,const QString &code)
{
}


void Protocol::outputNameChanged(int mid,unsigned output,const QString &name)
{
}


void Protocol::outputCrosspointChanged(int mid,unsigned output,unsigned input)
{
}


void Protocol::outputClipAlarmChanged(int mid,unsigned input,bool state)
{
}


void Protocol::outputSilenceAlarmChanged(int mid,unsigned input,bool state)
{
}


void Protocol::gpoChanged(int mid,unsigned output,const QString &code)
{
}


void Protocol::connectMatrices()
{
  for(unsigned i=0;i<proto_matrices->size();i++) {
    connect(proto_matrices->at(i),
	    SIGNAL(nodeStatusAlarm(int,unsigned,bool)),
	    this, SLOT(nodeStatusAlarmChanged(int,unsigned,bool)));
    connect(proto_matrices->at(i),
	    SIGNAL(inputNameChanged(int,unsigned,const QString &)),
	    this,SLOT(inputNameChanged(int,unsigned,const QString &)));
    connect(proto_matrices->at(i),
	    SIGNAL(inputClipAlarm(int,unsigned,bool)),
	    this,SLOT(inputClipAlarmChanged(int,unsigned,bool)));
    connect(proto_matrices->at(i),
	    SIGNAL(inputSilenceAlarm(int,unsigned,bool)),
	    this,SLOT(inputSilenceAlarmChanged(int,unsigned,bool)));
    connect(proto_matrices->at(i),
	    SIGNAL(gpiChanged(int,unsigned,const QString &)),
	    this,SLOT(gpiChanged(int,unsigned,const QString &)));
    connect(proto_matrices->at(i),
	    SIGNAL(outputNameChanged(int,unsigned,const QString &)),
	    this,SLOT(outputNameChanged(int,unsigned,const QString &)));
    connect(proto_matrices->at(i),
	    SIGNAL(outputCrosspointChanged(int,unsigned,unsigned)),
	    this,SLOT(outputCrosspointChanged(int,unsigned,unsigned)));
    connect(proto_matrices->at(i),
	    SIGNAL(outputClipAlarm(int,unsigned,bool)),
	    this,SLOT(outputClipAlarmChanged(int,unsigned,bool)));
    connect(proto_matrices->at(i),
	    SIGNAL(outputSilenceAlarm(int,unsigned,bool)),
	    this,SLOT(outputSilenceAlarmChanged(int,unsigned,bool)));
    connect(proto_matrices->at(i),
	    SIGNAL(gpoChanged(int,unsigned,const QString &)),
	    this,SLOT(gpoChanged(int,unsigned,const QString &)));
  }
}


Matrix *Protocol::matrix(int mid)
{
  int mtx=0;

  for(unsigned i=0;i<proto_matrices->size();i++) {
    mtx+=proto_matrices->at(i)->matrixQuantity();
    if(mtx>mid) {
      return proto_matrices->at(i);
    }
  }
  return NULL;
}
