// protocol.h
//
// Abstract base class for switcher control protocols.
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef PROTOCOL_H
#define PROTOCOL_H

#include <vector>

#include <QHostAddress>
#include <QObject>
#include <QString>

#include <sy/syprofile.h>

#include "matrix.h"

class Protocol : public QObject
{
 Q_OBJECT;
 public:
  enum Type {TypeSap=0,TypePath=1,TypeLast=2};
  Protocol(Type type,std::vector<Matrix *> *matrices,SyProfile *config,
	   QObject *parent=0);
  Protocol::Type type() const;
  static QString keyword(Type type);

 public slots:
  virtual void setReady(bool state)=0;
  virtual void nodeStatusAlarmChanged(int mid,unsigned node,bool state);
  virtual void inputNameChanged(int mid,unsigned input,const QString &name);
  virtual void inputClipAlarmChanged(int mid,unsigned input,bool state);
  virtual void inputSilenceAlarmChanged(int mid,unsigned input,bool state);
  virtual void gpiChanged(int mid,unsigned input,const QString &code);
  virtual void outputNameChanged(int mid,unsigned output,const QString &name);
  virtual void outputCrosspointChanged(int mid,unsigned output,unsigned input);
  virtual void outputClipAlarmChanged(int mid,unsigned input,bool state);
  virtual void outputSilenceAlarmChanged(int mid,unsigned input,bool state);
  virtual void gpoChanged(int mid,unsigned output,const QString &code);

 protected:
  virtual void connectMatrices();
  Matrix *matrix(int mid);

 private:
  Protocol::Type proto_type;
  std::vector <Matrix *> *proto_matrices;
};


#endif  // PROTOCOL_H
