// protocol_path.cpp
//
// Software Authority Protocol
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <sy/syrouting.h>

#include "protocol_path.h"

ProtocolPath::ProtocolPath(std::vector<Matrix *> *matrices,SyProfile *config,
			 QObject *parent)
  : Protocol(Protocol::TypePath,matrices,config,parent)
{
  path_server=new ServerPath(this);

  connect(path_server,SIGNAL(sendMatrixNames(int)),
	  this,SLOT(pathSendMatrixNamesData(int)));
  connect(path_server,SIGNAL(sendNodeNames(int,unsigned)),
	  this,SLOT(pathSendNodeNamesData(int,unsigned)));
  connect(path_server,SIGNAL(sendInputNames(int,unsigned)),
	  this,SLOT(pathSendInputNamesData(int,unsigned)));
  connect(path_server,SIGNAL(sendOutputNames(int,unsigned)),
	  this,SLOT(pathSendOutputNamesData(int,unsigned)));
  connect(path_server,SIGNAL(setRoute(int,unsigned,unsigned,unsigned)),
	  this,SLOT(pathSetRouteData(int,unsigned,unsigned,unsigned)));
  connect(path_server,SIGNAL(sendGpiInfo(int,unsigned,int)),
	  this,SLOT(pathSendGpiInfoData(int,unsigned,int)));
  connect(path_server,SIGNAL(sendGpoInfo(int,unsigned,int)),
	  this,SLOT(pathSendGpoInfoData(int,unsigned,int)));
  connect(path_server,
	  SIGNAL(setGpiState(int,unsigned,unsigned,int,const QString &)),
	  this,SLOT(pathSetGpiStateData(int,unsigned,unsigned,int,
				       const QString &)));
  connect(path_server,
	  SIGNAL(setGpoState(int,unsigned,unsigned,int,const QString &)),
	  this,SLOT(pathSetGpoStateData(int,unsigned,unsigned,int,
				       const QString &)));

  connectMatrices();
}


void ProtocolPath::setReady(bool state)
{
  path_server->setReady(state);
}


void ProtocolPath::nodeStatusAlarmChanged(int mid,unsigned node,bool state)
{
  path_server->send(QString().sprintf("NS\t%d\t%u\t%u\r\n",mid+1,node,state));
}


void ProtocolPath::inputNameChanged(int mid,unsigned input,const QString &name)
{
}


void ProtocolPath::inputClipAlarmChanged(int mid,unsigned input,bool state)
{
  path_server->send(QString().sprintf("CI\t%d\t%u\t%u\r\n",mid+1,input,state));
}


void ProtocolPath::inputSilenceAlarmChanged(int mid,unsigned input,bool state)
{
  path_server->send(QString().sprintf("SI\t%d\t%u\t%u\r\n",mid+1,input,state));
}


void ProtocolPath::gpiChanged(int mid,unsigned input,const QString &code)
{
  path_server->
    send(QString().sprintf("GS\t%d\tI\t%u\t",mid+1,input)+code+"\r\n");
}


void ProtocolPath::outputNameChanged(int mid,unsigned output,const QString &name)
{
}


void ProtocolPath::outputCrosspointChanged(int mid,unsigned output,unsigned input)
{
  path_server->send(QString().sprintf("ST\t%d\t%u\t%u\r\n",mid+1,input,output));
}


void ProtocolPath::outputClipAlarmChanged(int mid,unsigned output,bool state)
{
  path_server->send(QString().sprintf("CO\t%d\t%u\t%u\r\n",mid+1,output,state));
}


void ProtocolPath::outputSilenceAlarmChanged(int mid,unsigned output,bool state)
{
  path_server->send(QString().sprintf("SO\t%d\t%u\t%u\r\n",mid+1,output,state));
}


void ProtocolPath::gpoChanged(int mid,unsigned output,const QString &code)
{
  path_server->
    send(QString().sprintf("GS\t%d\tO\t%u\t",mid+1,output)+code+"\r\n");
}


void ProtocolPath::pathSendMatrixNamesData(int id)
{
  int mid=0;
  Matrix *mtx=NULL;

  while((mtx=matrix(mid))!=NULL) {
    path_server->send(QString().sprintf("MN\t%d",mid+1)+
		     "\t"+mtx->matrixName(mid)+"\r\n");
    mid++;
  }
}


void ProtocolPath::pathSendNodeNamesData(int id,unsigned mid)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    for(unsigned i=0;i<mtx->nodeQuantity(mid);i++) {
      path_server->
	send(QString().sprintf("NN\t%u",mid+1)+
	     "\t"+QString().sprintf("%u",i+1)+
	     "\t"+QString().sprintf("%u",mtx->nodeOnline(mid,i))+
	     "\t"+mtx->nodeHostName(mid,i)+
	     "\t"+mtx->nodeHostAddress(mid,i).toString()+
	     "\t"+mtx->nodeDeviceName(mid,i)+
	     "\r\n",id);
    }
  }
}


void ProtocolPath::pathSendInputNamesData(int id,unsigned mid)
{
  QString name;
  QString lwnum;
  QString lwaddr;
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    for(unsigned i=0;i<mtx->inputQuantity(mid);i++) {
      name=mtx->inputName(mid,i);
      if(name.isEmpty()) {
	name=QString().sprintf("SRC %u",mtx->inputSlotNumber(mid,i)+1);
      }
      if(SyRouting::livewireNumber(mtx->inputSourceAddress(mid,i))==0) {
	lwnum=SyRouting::sourceString(mtx->inputSourceAddress(mid,i),
				      mtx->inputSourceSlot(mid,i));
	lwaddr="0";
      }
      else {
	lwnum=QString().sprintf("%u",
		SyRouting::livewireNumber(mtx->inputSourceAddress(mid,i)));
	lwaddr=mtx->inputSourceAddress(mid,i).toString();
      }
      path_server->
	send(QString().sprintf("IN\t%u",mid+1)+
	     QString().sprintf("\t%u",i+1)+
	     "\t"+mtx->inputName(mid,i)+
	     "\t"+mtx->inputHostName(mid,i)+
	     "\t"+mtx->inputHostAddress(mid,i).toString()+
	     "\t"+QString().sprintf("%u",mtx->inputSlotNumber(mid,i)+1)+
	     "\t"+lwnum+
	     "\t"+lwaddr+
	     "\r\n",id);
    }
  }
}


void ProtocolPath::pathSendOutputNamesData(int id,unsigned mid)
{
  QString name;
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    for(unsigned i=0;i<mtx->outputQuantity(mid);i++) {
      name=mtx->outputName(mid,i);
      if(name.isEmpty()) {
	name=QString().sprintf("DST %u",mtx->outputSlotNumber(mid,1)+1);
      }
      path_server->
	send(QString().sprintf("ON\t%u",mid+1)+
	     "\t"+QString().sprintf("%u",i+1)+
	     "\t"+name+
	     "\t"+mtx->outputHostAddress(mid,i).toString()+
	     "\t"+mtx->outputHostName(mid,i)+
	     "\t"+QString().sprintf("%u",mtx->outputSlotNumber(mid,i)+1)+
	     "\r\n",id);
    }
  }
}


void ProtocolPath::pathSetRouteData(int id,unsigned mid,unsigned input,
				 unsigned output)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    mtx->setOutputCrosspoint(mid,output,input);
  }
}


void ProtocolPath::pathSendGpiInfoData(int id,unsigned cardnum,int input)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(cardnum))!=NULL) {
    if(input<0) {
      for(unsigned i=0;i<mtx->gpiQuantity(cardnum);i++) {
	path_server->send(QString().sprintf("GS\t%u\tI\t%d\t",cardnum+1,i+1)+
			 mtx->gpiCode(cardnum,i).toLower()+"\r\n",id);
      }
    }
    else {
      path_server->send(QString().sprintf("GS\t%u\tI\t%d\t",cardnum+1,input)+
		       mtx->gpiCode(cardnum,input-1).toLower()+"\r\n",id);
    }
  }
}


void ProtocolPath::pathSendGpoInfoData(int id,unsigned cardnum,int output)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(cardnum))!=NULL) {
    if(output<0) {
      for(unsigned i=0;i<mtx->gpiQuantity(cardnum);i++) {
	path_server->send(QString().sprintf("GS\t%u\tO\t%d\t",cardnum+1,i+1)+
			 mtx->gpoCode(cardnum,i).toLower()+"\r\n",id);
      }
    }
    else {
      path_server->send(QString().sprintf("GS\t%u\tO\t%d\t",cardnum+1,output)+
		       mtx->gpoCode(cardnum,output-1).toLower()+"\r\n",id);
    }
  }
}


void ProtocolPath::pathSetGpiStateData(int id,unsigned mid,unsigned input,
				     int msecs,const QString &code)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    mtx->setGpiCode(mid,input,msecs,code);
  }
}


void ProtocolPath::pathSetGpoStateData(int id,unsigned mid,unsigned output,
				     int msecs,const QString &code)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    mtx->setGpoCode(mid,output,msecs,code);
  }
}
