// protocol_path.h
//
// LWPath Protocol
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef PROTOCOL_PATH_H
#define PROTOCOL_PATH_H

#include "protocol.h"
#include "server_path.h"

class ProtocolPath : public Protocol
{
 Q_OBJECT;
 public:
 ProtocolPath(std::vector<Matrix *> *matrices,SyProfile *config,
	      QObject *parent=0);

 public slots:
  void setReady(bool state);
  void nodeStatusAlarmChanged(int mid,unsigned node,bool state);
  void inputNameChanged(int mid,unsigned input,const QString &name);
  void inputClipAlarmChanged(int mid,unsigned input,bool state);
  void inputSilenceAlarmChanged(int mid,unsigned input,bool state);
  void gpiChanged(int mid,unsigned input,const QString &code);
  void outputNameChanged(int mid,unsigned output,const QString &name);
  void outputCrosspointChanged(int mid,unsigned output,unsigned input);
  void outputClipAlarmChanged(int mid,unsigned output,bool state);
  void outputSilenceAlarmChanged(int mid,unsigned output,bool state);
  void gpoChanged(int mid,unsigned output,const QString &code);

 private slots:
  void pathSendMatrixNamesData(int id);
  void pathSendNodeNamesData(int id,unsigned cardnum);
  void pathSendInputNamesData(int id,unsigned cardnum);
  void pathSendOutputNamesData(int id,unsigned cardnum);
  void pathSetRouteData(int id,unsigned cardnum,unsigned input,unsigned output);
  void pathSendGpiInfoData(int id,unsigned cardnum,int input);
  void pathSendGpoInfoData(int id,unsigned cardnum,int output);
  void pathSetGpiStateData(int id,unsigned cardnum,unsigned input,int msecs,
			  const QString &code);
  void pathSetGpoStateData(int id,unsigned cardnum,unsigned input,int msecs,
			  const QString &code);

 private:
  ServerPath *path_server;
};


#endif  // PROTOCOL_PATH_H
