// protocol_sap.cpp
//
// Software Authority Protocol
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <map>

#include <sy/syrouting.h>

#include "protocol_sap.h"

ProtocolSap::ProtocolSap(std::vector<Matrix *> *matrices,SyProfile *config,
			 QObject *parent)
  : Protocol(Protocol::TypeSap,matrices,config,parent)
{
  sap_server=new ServerSap(this);
  connect(sap_server,SIGNAL(sendMatrixNames(int)),
	  this,SLOT(sapSendMatrixNamesData(int)));
  connect(sap_server,SIGNAL(sendInputNames(int,unsigned)),
	  this,SLOT(sapSendInputNamesData(int,unsigned)));
  connect(sap_server,SIGNAL(sendOutputNames(int,unsigned)),
	  this,SLOT(sapSendOutputNamesData(int,unsigned)));
  connect(sap_server,SIGNAL(setRoute(int,unsigned,unsigned,unsigned)),
	  this,SLOT(sapSetRouteData(int,unsigned,unsigned,unsigned)));
  connect(sap_server,SIGNAL(sendRouteInfo(int,unsigned,unsigned)),
	  this,SLOT(sapSendRouteInfoData(int,unsigned,unsigned)));
  connect(sap_server,SIGNAL(sendGpiInfo(int,unsigned,int)),
	  this,SLOT(sapSendGpiInfoData(int,unsigned,int)));
  connect(sap_server,SIGNAL(sendGpoInfo(int,unsigned,int)),
	  this,SLOT(sapSendGpoInfoData(int,unsigned,int)));
  connect(sap_server,
	  SIGNAL(setGpiState(int,unsigned,unsigned,int,const QString &)),
	  this,SLOT(sapSetGpiStateData(int,unsigned,unsigned,int,
				       const QString &)));
  connect(sap_server,
	  SIGNAL(setGpoState(int,unsigned,unsigned,int,const QString &)),
	  this,SLOT(sapSetGpoStateData(int,unsigned,unsigned,int,
				       const QString &)));

  connectMatrices();
}


void ProtocolSap::setReady(bool state)
{
  sap_server->setReady(state);
}


void ProtocolSap::inputNameChanged(int mid,unsigned input,const QString &name)
{
}


void ProtocolSap::gpiChanged(int mid,unsigned input,const QString &code)
{
  sap_server->
    send(QString().sprintf("GPIStat %d %u ",mid+1,input)+code.toLower()+
	 "\r\n");
  sap_server->send(">>");
}


void ProtocolSap::outputNameChanged(int mid,unsigned output,const QString &name)
{
}


void ProtocolSap::outputCrosspointChanged(int mid,unsigned output,unsigned input)
{
  sap_server->send(QString().sprintf("RouteStat %u %u %u False\r\n",
				     mid+1,output,input));
  sap_server->send(">>");
}


void ProtocolSap::gpoChanged(int mid,unsigned output,const QString &code)
{
  sap_server->
    send(QString().sprintf("GPOStat %d %u ",mid+1,output)+code.toLower()+
	 "\r\n");
  sap_server->send(">>");
}


void ProtocolSap::sapSendMatrixNamesData(int id)
{
  int mid=0;
  Matrix *mtx=NULL;

  sap_server->send("Begin RouterNames\r\n");
  while((mtx=matrix(mid))!=NULL) {
    sap_server->send(QString().sprintf("    %d ",mid+1)+
		     mtx->matrixName(mid)+"\r\n");
    mid++;
  }
  sap_server->send("End RouterNames\r\n");
  sap_server->send(">>",id);
}


void ProtocolSap::sapSendInputNamesData(int id,unsigned mid)
{
  QString name;
  QString lwnum;
  QString lwaddr;
  Matrix *mtx=NULL;

  sap_server->send(QString().sprintf("Begin SourceNames - %u\r\n",mid+1),id);
  if((mtx=matrix(mid))!=NULL) {
    for(unsigned i=0;i<mtx->inputQuantity(mid);i++) {
      name=mtx->inputName(mid,i);
      if(name.isEmpty()) {
	name=QString().sprintf("SRC %u",mtx->inputSlotNumber(mid,i)+1);
      }
      if(SyRouting::livewireNumber(mtx->inputSourceAddress(mid,i))==0) {
	lwnum=SyRouting::sourceString(mtx->inputSourceAddress(mid,i),
				      mtx->inputSourceSlot(mid,i));
	lwaddr="0";
      }
      else {
	lwnum=QString().sprintf("%u",
		SyRouting::livewireNumber(mtx->inputSourceAddress(mid,i)));
	lwaddr=mtx->inputSourceAddress(mid,i).toString();
      }
      sap_server->
	send(QString().sprintf("    %u",i+1)+
	     "\t"+name+
	     "\t"+name+" ON "+mtx->inputHostName(mid,i)+
	     "\t"+mtx->inputHostAddress(mid,i).toString()+
	     "\t"+mtx->inputHostName(mid,i)+
	     "\t"+QString().sprintf("%u",mtx->inputSlotNumber(mid,i)+1)+
	     "\t"+lwnum+
	     "\t"+lwaddr+
	     "\r\n",id);
    }
  }
  sap_server->send(QString().sprintf("End SourceNames - %u\r\n",mid+1),id);
  sap_server->send(">>",id);
}


void ProtocolSap::sapSendOutputNamesData(int id,unsigned mid)
{
  QString name;
  Matrix *mtx=NULL;

  sap_server->send(QString().sprintf("Begin DestNames - %u\r\n",mid+1),id);
  if((mtx=matrix(mid))!=NULL) {
    for(unsigned i=0;i<mtx->outputQuantity(mid);i++) {
      name=mtx->outputName(mid,i);
      if(name.isEmpty()) {
	name=QString().sprintf("DST %u",mtx->outputSlotNumber(mid,1)+1);
      }
      sap_server->
	send(QString().sprintf("    %u",i+1)+
	     "\t"+name+
	     "\t"+name+" ON "+mtx->outputHostName(mid,i)+
	     "\t"+mtx->outputHostAddress(mid,i).toString()+
	     "\t"+mtx->outputHostName(mid,i)+
	     "\t"+QString().sprintf("%u",mtx->outputSlotNumber(mid,i)+1)+
	     "\r\n",id);
    }
  }
  sap_server->send(QString().sprintf("End DestNames - %u\r\n",mid+1),id);
  sap_server->send(">>",id);
}


void ProtocolSap::sapSetRouteData(int id,unsigned mid,unsigned input,
				 unsigned output)
{
  Matrix *mtx=NULL;

  sap_server->send("Route Change Initiated\r\n",id);
  sap_server->send(">>",id);
  if((mtx=matrix(mid))!=NULL) {
    mtx->setOutputCrosspoint(mid,output,input);
  }
}


void ProtocolSap::sapSendRouteInfoData(int id,unsigned cardnum,unsigned output)
{
  //  printf("sapSendRouteInfoData(%d,%u,%u)\n",id,cardnum,output);
  Matrix *mtx=NULL;

  if((mtx=matrix(cardnum-1))==NULL) {
    sap_server->send("Error - Router Does Not exist.\r\n",id);
  }
  else {
    if(output==0) {
      for(unsigned i=0;i<mtx->outputQuantity(cardnum-1);i++) {
	sap_server->
	  send(QString().sprintf("RouteStat %u %u %u False\r\n",cardnum,i+1,
				 mtx->outputCrosspoint(cardnum-1,i+1)),id);
      }
    }
    else {
      if(output<=mtx->outputQuantity(cardnum-1)) {
	sap_server->
	  send(QString().sprintf("RouteStat %u %u %u False\r\n",cardnum,output,
				 mtx->outputCrosspoint(cardnum-1,output)),id);
      }
      else {
	sap_server->send("Error - Output Does Not exist.\r\n",id);
      }
    }
  }
}


void ProtocolSap::sapSendGpiInfoData(int id,unsigned cardnum,int input)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(cardnum))==NULL) {
    sap_server->send("Error - Router Does Not exist.\r\n",id);
  }
  else {
    sap_server->send(">>",id);
    if(input<0) {
      for(unsigned i=0;i<mtx->gpiQuantity(cardnum);i++) {
	sap_server->send(QString().sprintf("GPIStat %u %d ",cardnum+1,i+1)+
			 mtx->gpiCode(cardnum,i).toLower()+"\r\n",id);
      }
    }
    else {
      sap_server->send(QString().sprintf("GPIStat %u %d ",cardnum+1,input)+
		       mtx->gpiCode(cardnum,input-1).toLower()+"\r\n",id);
    }
  }
}


void ProtocolSap::sapSendGpoInfoData(int id,unsigned cardnum,int output)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(cardnum))==NULL) {
    sap_server->send("Error - Router Does Not exist.\r\n",id);
  }
  else {
    sap_server->send(">>",id);
    if(output<0) {
      for(unsigned i=0;i<mtx->gpoQuantity(cardnum);i++) {
	sap_server->send(QString().sprintf("GPOStat %u %d ",cardnum+1,i+1)+
			 mtx->gpoCode(cardnum,i).toLower()+"\r\n",id);
      }
    }
    else {
      sap_server->send(QString().sprintf("GPOStat %u %d ",cardnum+1,output)+
		       mtx->gpoCode(cardnum,output-1).toLower()+"\r\n",id);
    }
  }
}


void ProtocolSap::sapSetGpiStateData(int id,unsigned mid,unsigned input,
				     int msecs,const QString &code)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    sap_server->send(">>",id);
    mtx->setGpiCode(mid,input,msecs,code);
  }
}


void ProtocolSap::sapSetGpoStateData(int id,unsigned mid,unsigned output,
				     int msecs,const QString &code)
{
  Matrix *mtx=NULL;

  if((mtx=matrix(mid))!=NULL) {
    sap_server->send(">>",id);
    mtx->setGpoCode(mid,output,msecs,code);
  }
}
