// server_path.cpp
//
// LWPath Protocol Server for lwpathd(8).
//
//   (C) Copyright 2015 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License version 2 as
//   published by the Free Software Foundation.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <stdio.h>

#include <QStringList>

#include "server_path.h"

ServerPath::ServerPath(QObject *parent)
  : ServerNet(SERVER_PATH_PORT,QHostAddress("239.192.255.3"),parent)
{
}


ServerPath::~ServerPath()
{
}


void ServerPath::processCommand(int id,const SyAString &cmd)
{
  //  printf("processCommand(%d,%s)\n",id,(const char *)cmd.toUtf8());

  unsigned cardnum;
  unsigned input;
  unsigned output;
  unsigned msecs=0;
  bool ok=false;
  QStringList cmds=cmd.split(" ");

  if(cmds[0]=="DC") {
    closeConnection(id);
  }

  if((cmds[0]=="GQ")&&(cmds.size()>=3)) {
    cardnum=cmds[1].toUInt(&ok);
    if(ok) {
      if(cmds.size()==3) {
	if(cmds[2].toLower()=="i") {
	  emit sendGpiInfo(id,cardnum-1,-1);
	}
	if(cmds[2].toLower()=="o") {
	  emit sendGpoInfo(id,cardnum-1,-1);
	}
      }
      else {
	input=cmds[3].toUInt(&ok);
	if(ok) {
	  if(cmds[2].toLower()=="i") {
	    emit sendGpiInfo(id,cardnum-1,input);
	  }
	  if(cmds[2].toLower()=="o") {
	    emit sendGpoInfo(id,cardnum-1,input);
	  }
	}
      }
    }
  }

  if(cmds[0]=="GS") {
    if(cmds.size()==6) {
      msecs=cmds[5].toUInt();
      cardnum=cmds[1].toUInt(&ok);
      if(ok) {
	output=cmds[3].toUInt(&ok);
	if(cmds[4].length()==5) {
	  if(cmds[2].toLower()=="i") {
	    emit setGpiState(id,cardnum-1,output-1,msecs,cmds[4]);
	  }
	  if(cmds[2].toLower()=="o") {
	    emit setGpoState(id,cardnum-1,output-1,msecs,cmds[4]);
	  }
	}
      }
    }
  }

  if((cmds[0]=="IN")&&(cmds.size()==2)) {
    cardnum=cmds[1].toUInt(&ok);
    if(ok) {
      emit sendInputNames(id,cardnum-1);
    }
  }

  if(cmds[0]=="MN") {
    emit sendMatrixNames(id);
  }

  if((cmds[0]=="NN")&&(cmds.size()==2)) {
    cardnum=cmds[1].toUInt(&ok);
    if(ok) {
      emit sendNodeNames(id,cardnum-1);
    }
  }

  if((cmds[0]=="ON")&&(cmds.size()==2)) {
    cardnum=cmds[1].toUInt(&ok);
    if(ok) {
      emit sendOutputNames(id,cardnum-1);
    }
  }

  if(cmds[0]=="ST") {
    if(cmds.size()==4) {
      cardnum=cmds[1].toUInt(&ok);
      if(ok) {
	input=cmds[2].toUInt(&ok);
	if(ok) {
	  output=cmds[3].toUInt(&ok);
	  if(ok) {
	    emit setRoute(id,cardnum-1,input-1,output-1);
	  }
	}
      }
    }
  }
}
