// connect_sap.h
//
// A connection driver for SoftwareAuthority Protocol
//
//   (C) Copyright 2016 Fred Gleason <fredg@paravelsystems.com>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as
//   published by the Free Software Foundation; either version 2 of
//   the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public
//   License along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef CONNECT_SAP_H
#define CONNECT_SAP_H

#include <stdint.h>

#include <map>
#include <vector>

#include <QList>
#include <QObject>
#include <QTcpSocket>
#include <QTimer>

#define CONNECTSAP_HOLDOFF_INTERVAL 5000

class ConnectSap : public QObject
{
 Q_OBJECT
 public:
  ConnectSap(QObject *parent=0);
  ~ConnectSap();
  unsigned inputQuantity() const;
  QString inputName(unsigned input);
  QString inputLongName(unsigned input);
  unsigned outputQuantity() const;
  QString outputName(unsigned output);
  QString outputLongName(unsigned output);
  unsigned outputCrosspoint(unsigned output);
  void setOutputCrosspoint(unsigned output,unsigned input);
  void connectToHost(unsigned matrix_num,const QString &hostname,uint16_t port,
		     const QString &username,const QString &passwd);
  void connectToHost(unsigned matrix_num,const QString &hostname,uint16_t port,
		     const QString &username,const QString &passwd,
		     const QList<unsigned> &outputs);

 signals:
  void connected(bool state);
  void inputListChanged();
  void outputListChanged();
  void outputCrosspointChanged(unsigned output,unsigned input);

 private slots:
  void connectedData();
  void connectionClosedData();
  void holdoffReconnectData();
  void readyReadData();
  void errorData(QAbstractSocket::SocketError err);

 private:
  void DispatchCommand(const QString &cmd);
  void ReadSourceName(const QString &cmd);
  void ReadDestName(const QString &cmd);
  void BubbleSort(std::map<unsigned,QString> *names,
		  std::vector<unsigned> *ptrs);
  void SendCommand(const QString &cmd);
  void MakeSocket();
  QTcpSocket *sap_socket;
  QString sap_hostname;
  uint16_t sap_port;
  QString sap_username;
  QString sap_password;
  unsigned sap_matrix_number;
  QString sap_accum;
  bool sap_reading_sources;
  bool sap_reading_dests;
  std::vector<QString> sap_input_names;
  std::vector<QString> sap_input_long_names;
  std::vector<QString> sap_output_names;
  std::vector<QString> sap_output_long_names;
  std::map<unsigned,unsigned> sap_output_xpoints;
  QList<unsigned> sap_active_outputs;
  QTimer *sap_holdoff_timer;
};


#endif  // CONNECT_SAP_H
